<?php

declare(strict_types=1);

namespace Drupal\lms_messages\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\lms_messages\MessageService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * LMS Messages settings form class.
 */
final class LmsMessagesSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * Service setter.
   */
  public function setEntityTypeManager(EntityTypeManagerInterface $entity_type_manager): void {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    $instance = parent::create($container);
    $instance->setEntityTypeManager($container->get('entity_type.manager'));
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'lms_messages_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [MessageService::CONFIG_NAME];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(MessageService::CONFIG_NAME);
    $template_behavior_settings = $config->get('template_behavior');
    $form['template_behavior'] = [
      '#tree' => TRUE,
      '#type' => 'fieldset',
      '#title' => $this->t('Message template behavior'),
      '#description' => $this->t('Control additional behavior when Messages with certain templates are created.'),
    ];
    /** @var \Drupal\Core\Config\Entity\ConfigEntityInterface $template */
    foreach ($this->entityTypeManager->getStorage('message_template')->loadByProperties(['template' => \array_values(MessageService::SUPPORTED_TEMPLATES)]) as $template_id => $template) {
      $defaults = \is_array($template_behavior_settings) && \array_key_exists($template_id, $template_behavior_settings) ? $template_behavior_settings[$template_id] : [];
      $form['template_behavior'][$template_id] = [
        '#type' => 'details',
        '#title' => $template->label(),
        '#open' => $template->status(),
      ];
      $form['template_behavior'][$template_id]['status'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enabled'),
        '#default_value' => $template->status(),
      ];
      $form['template_behavior'][$template_id]['email'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('E-mail'),
        '#description' => $this->t('Send messages using this template as e-mails.'),
        '#default_value' => \array_key_exists('email', $defaults) ? $defaults['email'] : FALSE,
      ];
      $form['template_behavior'][$template_id]['prevent_message_save'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Prevent message save'),
        '#description' => $this->t('Do not save message entity after sending. NOTE: make sure that this type of messages is not used anywhere else.'),
        '#default_value' => \array_key_exists('prevent_message_save', $defaults) ? $defaults['prevent_message_save'] : FALSE,
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->config(MessageService::CONFIG_NAME);
    /** @var \Drupal\Core\Config\Entity\ConfigEntityInterface[] */
    $templates = $this->entityTypeManager->getStorage('message_template')->loadByProperties([
      'template' => \array_values(MessageService::SUPPORTED_TEMPLATES),
    ]);
    $template_behavior_settings = [];
    foreach ($form_state->getValue('template_behavior') as $template_id => $settings) {
      if (!\array_key_exists($template_id, $templates)) {
        continue;
      }
      $status = (bool) $settings['status'];
      unset($settings['status']);
      if ($templates[$template_id]->status() !== $status) {
        $templates[$template_id]->setStatus($status)->save();
      }
      $template_behavior_settings[$template_id] = $settings;
    }
    $config->set('template_behavior', $template_behavior_settings)->save();

    parent::submitForm($form, $form_state);
  }

}
