<?php

declare(strict_types=1);

namespace Drupal\lms_messages\Hook;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\comment\CommentInterface;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\lms\Entity\AnswerInterface;
use Drupal\lms\Entity\CourseStatusInterface;
use Drupal\lms_answer_comments\Hook\AnswerCommentsHooks;
use Drupal\lms_messages\GroupMembershipRequestHelper;
use Drupal\lms_messages\MessageService;
use Symfony\Component\DependencyInjection\Attribute\Autowire;
use Symfony\Component\DependencyInjection\Attribute\AutowireCallable;

/**
 * Contains message-related hooks.
 */
final class MessageHooks {

  public function __construct(
    #[AutowireCallable(service: ModuleHandlerInterface::class, method: 'moduleExists', lazy: TRUE)]
    protected readonly \Closure $moduleExists,
    #[Autowire(service: MessageService::class, lazy: TRUE)]
    protected readonly MessageService $messageService,
    #[Autowire(service: RouteMatchInterface::class, lazy: TRUE)]
    protected readonly RouteMatchInterface $currentRoute,
  ) {}

  #[Hook('comment_insert')]
  public function commentInsert(CommentInterface $comment): void {
    if (
      $comment->isSyncing() ||
      $comment->bundle() !== AnswerCommentsHooks::COMMENT_TYPE ||
      $this->currentRoute->getRouteName() !== 'lms.answer.details'
    ) {
      return;
    }
    $answer = $this->currentRoute->getParameter('lms_answer');
    \assert($answer instanceof AnswerInterface);

    // No notification if the student added the comment.
    if ($answer->getOwnerId() === $comment->getOwnerId()) {
      return;
    }

    $this->messageService->newAnswerComment($answer, $comment);
  }

  #[Hook('group_relationship_insert')]
  public function groupRelationshipInsert(GroupRelationshipInterface $group_relationship): void {
    if ($group_relationship->isSyncing()) {
      return;
    }

    // New membership request created.
    if (($this->moduleExists)('lms_membership_request')) {
      // Use an external class so we don't have a hard dependency here.
      $message_parameters = GroupMembershipRequestHelper::getNewRelationshipParameters($group_relationship);
      if (\count($message_parameters) !== 0) {
        $this->messageService->createFromParameters($message_parameters);
        return;
      }
    }
  }

  #[Hook('group_relationship_update')]
  public function groupRelationshipUpdate(GroupRelationshipInterface $group_relationship): void {
    if ($group_relationship->isSyncing()) {
      return;
    }

    // Membership request has been updated.
    if (($this->moduleExists)('lms_membership_request')) {
      // Use an external class so we don't have a hard dependency here.
      $message_parameters = GroupMembershipRequestHelper::getUpdatedRelationshipParameters($group_relationship);
      if (\count($message_parameters) !== 0) {
        $this->messageService->createFromParameters($message_parameters);
        return;
      }
    }
  }

  #[Hook('lms_course_status_update')]
  public function lmsCourseStatusUpdate(CourseStatusInterface $course_status): void {
    if ($course_status->isSyncing()) {
      return;
    }

    // Only act if there has been a needs evaluation -> passed or failed or
    // any -> needs evaluation status change.
    $status = $course_status->getStatus();
    if ($status === CourseStatusInterface::STATUS_PROGRESS) {
      return;
    }
    // @todo https://www.drupal.org/node/3295826
    if (!$course_status->original instanceof CourseStatusInterface) {
      return;
    }

    $previous_status = $course_status->original->getStatus();
    if (
      $status === CourseStatusInterface::STATUS_NEEDS_EVALUATION &&
      $previous_status !== CourseStatusInterface::STATUS_NEEDS_EVALUATION
    ) {
      $this->messageService->courseNeedsEvaluation($course_status);
    }
    elseif (
      $previous_status === CourseStatusInterface::STATUS_NEEDS_EVALUATION &&
      \in_array($status, [CourseStatusInterface::STATUS_PASSED, CourseStatusInterface::STATUS_FAILED], TRUE)
    ) {
      $this->messageService->courseEvaluated($course_status);
    }
  }

}
