<?php

declare(strict_types=1);

namespace Drupal\lms_messages;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\comment\CommentInterface;
use Drupal\lms\Entity\AnswerInterface;
use Drupal\lms\Entity\CourseStatusInterface;
use Symfony\Component\DependencyInjection\Attribute\AutowireCallable;

/**
 * General message related logic.
 */
class MessageService {

  public const SUPPORTED_TEMPLATES = [
    'course_evaluation' => 'lms_messages_course_evaluation',
    'course_evaluated' => 'lms_messages_course_evaluated',
    'new_request' => 'lms_messages_new_request',
    'approved_request' => 'lms_messages_approved_request',
    'rejected_request' => 'lms_messages_rejected_request',
    'new_answer_comment' => 'lms_messages_answer_comment',
  ];

  public const CONFIG_NAME = 'lms_messages.settings';

  public function __construct(
    #[AutowireCallable(service: EntityTypeManagerInterface::class, method: 'getStorage')]
    private \Closure $getStorage,
    #[AutowireCallable(service: ConfigFactoryInterface::class, method: 'get', lazy: TRUE)]
    private \Closure $getConfig,
    #[AutowireCallable(service: 'message_notify.sender', method: 'send', lazy: TRUE)]
    private \Closure $sendMessage,
  ) {}

  /**
   * Teacher just added a new comment to a student answer.
   */
  public function newAnswerComment(AnswerInterface $answer, CommentInterface $comment): void {
    $template = ($this->getStorage)('message_template')->load(self::SUPPORTED_TEMPLATES['new_answer_comment']);
    if ($template === NULL || !$template->status()) {
      return;
    }
    $this->createMessage([
      'template' => $template->id(),
      'uid' => $answer->getOwnerId(),
      'course' => $answer->getLessonStatus()->getCourseStatus()->getCourse(),
      'activity' => $answer->getActivity()->id(),
      'comment' => $comment->id(),
    ]);
  }

  /**
   * Course has just been evaluated, create a message for the student.
   */
  public function courseEvaluated(CourseStatusInterface $course_status): void {
    $template = ($this->getStorage)('message_template')->load(self::SUPPORTED_TEMPLATES['course_evaluated']);
    if ($template === NULL || !$template->status()) {
      return;
    }

    $this->createMessage([
      'template' => $template->id(),
      'uid' => $course_status->getUserId(),
      'course' => $course_status->getCourseId(),
    ]);
  }

  /**
   * Course needs evaluating, create a message for the teacher(s).
   */
  public function courseNeedsEvaluation(CourseStatusInterface $course_status): void {
    $template = ($this->getStorage)('message_template')->load(self::SUPPORTED_TEMPLATES['course_evaluation']);
    if ($template === NULL || !$template->status()) {
      return;
    }

    $course = $course_status->getCourse();
    foreach ($course->getMembers() as $member) {
      $account = $member->getUser();
      if ($account->isAnonymous()) {
        continue;
      }

      $this->createMessage([
        'template' => $template->id(),
        'uid' => $account->id(),
        'course' => $course_status->getCourseId(),
        'user' => $course_status->getUserId(),
      ]);
    }
  }

  /**
   * Create messages from parameters.
   */
  public function createFromParameters(array $message_parameters): void {
    if (\count($message_parameters) === 0) {
      return;
    }
    $template = ($this->getStorage)('message_template')->load($message_parameters[0]['template']);
    if ($template === NULL || !$template->status()) {
      return;
    }

    foreach ($message_parameters as $parameters) {
      $this->createMessage($parameters);
    }
  }

  /**
   * Create a message helper.
   */
  private function createMessage(array $properties): void {
    $message = (($this->getStorage)('message'))->create($properties);

    $template_behavior = ($this->getConfig)(self::CONFIG_NAME)->get('template_behavior');

    if (!\is_array($template_behavior)) {
      $message->save();
      return;
    }

    $template_id = $message->getTemplate()->id();
    if (!\array_key_exists($template_id, $template_behavior)) {
      $message->save();
      return;
    }

    // Send an email if configured.
    if ($template_behavior[$template_id]['email'] === TRUE) {
      ($this->sendMessage)($message);
    }

    // Save message if not specified otherwise.
    if ($template_behavior[$template_id]['prevent_message_save'] !== TRUE) {
      $message->save();
    }
  }

}
