<?php

declare(strict_types=1);

namespace Drupal\Tests\lms_messages\Kernel;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\KernelTests\KernelTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\grequest\Plugin\Group\Relation\GroupMembershipRequest;
use Drupal\lms\Entity\CourseStatusInterface;
use Drupal\lms_answer_comments\Hook\AnswerCommentsHooks;
use Drupal\lms_messages\MessageService;
use Drupal\message\MessageInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Tests creation of messages.
 *
 * @group lms_messages
 */
final class LmsMessagesTest extends KernelTestBase {

  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'field',
    'filter',
    'comment',
    'flexible_permissions',
    'group',
    'grequest',
    'lms',
    'lms_answer_comments',
    'lms_membership_request',
    'lms_messages',
    'message',
    'message_notify',
    'options',
    'state_machine',
    'text',
    'user',
    'views',
  ];

  /**
   * Entity type manager.
   */
  private EntityTypeManagerInterface $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installEntitySchema('action');
    $this->installEntitySchema('user');
    $this->installEntitySchema('group');
    $this->installEntitySchema('group_relationship');
    $this->installEntitySchema('group_config_wrapper');
    $this->installEntitySchema('message');
    $this->installEntitySchema('lms_course_status');
    $this->installEntitySchema('lms_lesson_status');
    $this->installEntitySchema('lms_activity');
    $this->installEntitySchema('lms_answer');
    $this->installEntitySchema('comment');
    $this->installConfig(['filter']);
    $this->installConfig(['group']);
    $this->installConfig(['grequest']);
    $this->installConfig(['lms']);
    $this->installConfig(['lms_membership_request']);
    $this->installConfig('comment');
    $this->installConfig(['lms_answer_comments']);
    $this->installConfig(['lms_messages']);
    $this->installSchema('comment', ['comment_entity_statistics']);

    $this->entityTypeManager = $this->container->get(EntityTypeManagerInterface::class);
  }

  /**
   * Tests creation of messages.
   */
  public function testLmsMessages(): void {
    $teacher = $this->createUser();
    /** @var \Drupal\lms\Entity\Bundle\Course */
    $course = $this->entityTypeManager->getStorage('group')->create([
      'type' => 'lms_course',
      'name' => 'Test course',
      'uid' => $teacher->id(),
    ]);
    $course->save();
    $course->addMember($teacher);
    $student = $this->createUser();

    // Course needs evaluation.
    /** @var \Drupal\lms\Entity\CourseStatusInterface */
    $course_status = $this->entityTypeManager->getStorage('lms_course_status')->create([
      'uid' => $student->id(),
      'gid' => $course->id(),
      'score' => 0,
      'status' => CourseStatusInterface::STATUS_PROGRESS,
    ]);
    $course_status->save();
    $course_status->setStatus(CourseStatusInterface::STATUS_NEEDS_EVALUATION);
    $course_status->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), MessageService::SUPPORTED_TEMPLATES['course_evaluation']);
    self::assertEquals($message->get('course')->target_id, $course->id());
    self::assertEquals($message->get('user')->target_id, $student->id());
    self::assertEquals($message->get('uid')->target_id, $teacher->id());

    // Course evaluated.
    $course_status->setStatus(CourseStatusInterface::STATUS_PASSED);
    $course_status->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), MessageService::SUPPORTED_TEMPLATES['course_evaluated']);
    self::assertEquals($message->get('course')->target_id, $course->id());
    self::assertEquals($message->get('uid')->target_id, $student->id());

    // New membership request.
    $course->removeMember($student);
    /** @var \Drupal\group\Entity\GroupRelationshipInterface */
    $membership_request = $this->entityTypeManager->getStorage('group_relationship')->create([
      'gid' => $course->id(),
      'entity_id' => $student->id(),
      'label' => 'membership request',
      'uid' => $student->id(),
      // From lms_membership_request config.
      'type' => 'lms_course-996ae0cbc1a35b89222ab',
      'plugin_id' => 'group_membership_request',
      GroupMembershipRequest::STATUS_FIELD => GroupMembershipRequest::REQUEST_NEW,
    ]);
    $membership_request->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), MessageService::SUPPORTED_TEMPLATES['new_request']);
    self::assertEquals($message->get('course')->target_id, $course->id());
    self::assertEquals($message->get('user')->target_id, $student->id());
    self::assertEquals($message->get('uid')->target_id, $teacher->id());

    // Membership request approved.
    $membership_request->set(GroupMembershipRequest::STATUS_FIELD, GroupMembershipRequest::REQUEST_APPROVED)->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), MessageService::SUPPORTED_TEMPLATES['approved_request']);
    self::assertEquals($message->get('course')->target_id, $course->id());
    self::assertEquals($message->get('uid')->target_id, $student->id());

    // Answer comment message test.
    $this->entityTypeManager->getStorage('lms_activity_type')->create([
      'id' => 'no_answer',
      'name' => 'No answer',
      'pluginId' => 'no_answer',
    ])->save();
    $activity = $this->entityTypeManager->getStorage('lms_activity')->create([
      'uid' => $teacher->id(),
      'name' => 'Test activity',
      'status' => TRUE,
      'type' => 'no_answer',
    ]);
    $activity->save();
    $lesson_status = $this->entityTypeManager->getStorage('lms_lesson_status')->create([
      'course_status' => $course_status->id(),
    ]);
    $lesson_status->save();
    $answer = $this->entityTypeManager->getStorage('lms_answer')->create([
      'user_id' => $student->id(),
      'activity' => $activity->id(),
      'lesson_status' => $lesson_status->id(),
    ]);
    $answer->save();

    // To trigger creation of this message we need to be on the correct route.
    $request = Request::create(Url::fromRoute('lms.answer.details', [
      'lms_answer' => $answer->id(),
      'js' => 'nojs',
    ])->toString());
    $this->container->get(HttpKernelInterface::class)->handle($request);

    $comment = $this->entityTypeManager->getStorage('comment')->create([
      'comment_type' => AnswerCommentsHooks::COMMENT_TYPE,
      'entity_id' => $answer->id(),
      'entity_type' => 'lms_answer',
      'field_name' => AnswerCommentsHooks::FIELD_NAME,
      'status' => TRUE,
      'uid' => $teacher->id(),
      'subject' => 'Some subject',
      'comment_body' => 'Some comment',
    ]);
    $comment->save();
    $message = $this->getLastMessage();
    self::assertEquals($message->bundle(), MessageService::SUPPORTED_TEMPLATES['new_answer_comment']);
    self::assertEquals($message->get('comment')->target_id, $comment->id());
    self::assertEquals($message->get('activity')->target_id, $activity->id());
    self::assertEquals($message->get('course')->target_id, $course->id());
  }

  /**
   * Get last message created.
   */
  private function getLastMessage(): MessageInterface {
    $message_storage = $this->entityTypeManager->getStorage('message');
    $results = $message_storage->getQuery()
      ->accessCheck(FALSE)
      ->sort('mid', 'DESC')
      ->range(0, 1)
      ->execute();
    self::assertEquals(1, \count($results));
    $message = $message_storage->load(\reset($results));
    \assert($message instanceof MessageInterface);
    return $message;
  }

}
